@@ -4,6 +4,12 @@
 from enum import Enum, auto
 
 
+class TrafficRule(Enum):
+    """ Enum for MarkRule
+    """
+    RTH = auto()
+    LTH = auto()
+    none = auto()
 
 class MarkRule(Enum):
     """ Enum for MarkRule
@@ -21,6 +27,7 @@ class LaneType(Enum):
     shoulder = auto()
     biking = auto()
     sidewalk = auto()
+    curb = auto()
     border = auto()
     restricted = auto()
     parking = auto()
@@ -166,4 +173,4 @@ class JunctionGroupType(Enum):
     """ Enum for JunctionGroup
     """
     roundabout = auto()
-    unknown = auto()
\ No newline at end of file
+    unknown = auto()
